/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.imda;

import java.io.Writer;
import java.util.List;

import org.tinygroup.context.Context;
import org.tinygroup.imda.config.ModelDefine;
import org.tinygroup.imda.config.ModelProcessorDefine;

/**
 * 模型管理器<br>
 * 所谓模型，就是含有一些描述信息的对象，通过对此模型进行解释，就可以动态完成一些系统资源的内容
 * 
 * @author luog
 * 
 */
public interface ModelManager {
	String MODELMANAGER_BEAN = "modelManager";

	/**
	 * 更新模型<br>
	 * 当模型被修改之后，通过模型更新可以使之生效，如果模型不存在，则相当于添加模型
	 * 
	 * @param model
	 */
	void updateModel(Object model);

	/**
	 * 添加模型，如果已经存在，则相当于更新模型
	 * @param model
	 */
	void addModel(Object model);

	/**
	 * 删除模型
	 * 
	 * @param model
	 */
	void removeModel(Object model);

	/**
	 * 增加模型处理器
	 * 
	 * @param modelType
	 * @param modelProcessor
	 */
	void addModelDefine(ModelDefine modelDefine);

	/**
	 * 获取模型
	 * 
	 * @param modelId
	 * @return
	 */
	Object getModel(String modelId);

	/**
	 * 获取模型定义
	 * 
	 * @param name
	 * @return
	 */
	ModelDefine getModelDefine(String modelDefineId);

	/**
	 * 获取模型定义处理
	 * 
	 * @param modelDefineId
	 * @param processorName
	 * @return
	 */
	ModelProcessorDefine getModelProcessorDefine(String modelDefineId,
			String processorName);

	/**
	 * 获取模型定义处理
	 * 
	 * @param modelDefine
	 * @param processorName
	 * @return
	 */
	ModelProcessorDefine getModelProcessorDefine(ModelDefine modelDefine,
			String processorName);

	/**
	 * 对模型服务进行处理
	 * 
	 * @param model
	 * @param processorName
	 * @return
	 */
	void processService(String modelId, String processorName, String stageName,
			Context context);

	/**
	 * 对模型视图进行处理
	 * 
	 * @param modelId
	 * @param processorName
	 * @param writer
	 */
	void processView(String modelId, String processorName, String stageName,
			Writer writer, Context context);

	/**
	 * 处理参数
	 * 
	 * @param model
	 * @param processorName
	 * @param writer
	 */
	void processParameter(String modelId, String processorName,
			String stageName, Context context);

	List<ModelLoader> getModelLoaders();
}
