/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.imda.impl;

import java.io.Writer;
import java.util.List;

import org.tinygroup.commons.tools.StringUtil;
import org.tinygroup.context.Context;
import org.tinygroup.imda.ModelLoader;
import org.tinygroup.imda.ModelManager;
import org.tinygroup.imda.config.ModelDefine;
import org.tinygroup.imda.config.ModelProcessorDefine;
import org.tinygroup.imda.config.ModelProcessorStage;
import org.tinygroup.imda.processor.ModelServiceProcessor;
import org.tinygroup.imda.processor.ModelViewProcessor;
import org.tinygroup.imda.processor.ParameterBuilder;
import org.tinygroup.springutil.SpringUtil;

public class ModelManagerImpl implements ModelManager {

	private ModelContainer container = new ModelContainer();

	public void updateModel(Object model) {
		container.updateModel(model);
	}

	public void addModel(Object model) {
		container.addModel(model);
	}

	public void removeModel(Object model) {
		container.removeModel(model);
	}

	public void addModelDefine(ModelDefine modelDefine) {
		container.addModelDefine(modelDefine);
	}

	public Object getModel(String modelId) {
		return container.getModel(modelId);
	}

	public ModelDefine getModelDefine(String modelDefineId) {
		return container.getModelDefine(modelDefineId);
	}

	public ModelProcessorDefine getModelProcessorDefine(
			String modelDefineId, String processorName) {
		ModelDefine modelDefine = getModelDefine(modelDefineId);
		return container.getModelProcessorDefine(modelDefine, processorName);
	}

	public ModelProcessorDefine getModelProcessorDefine(
			ModelDefine modelDefine, String processorName) {
		return getModelProcessorDefine(modelDefine.getId(), processorName);
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	public void processService(String modelId,  String processorName,
			String stageName, Context context) {
		Object model = getModel(modelId);
		ModelProcessorStage processorStage = container.getModelProcessorStage(
				model, processorName, stageName);
		String serviceProcessorBean = processorStage.getServiceProcessorBean();
		if (StringUtil.isBlank(serviceProcessorBean)) {
			return;
		}
		ModelServiceProcessor serviceProcessor = SpringUtil
				.getBean(serviceProcessorBean);
		serviceProcessor.process(model, context);
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	public void processView(String modelId,  String processorName,
			String stageName, Writer writer, Context context) {
		Object model = getModel(modelId);
		ModelProcessorStage processorStage = container.getModelProcessorStage(
				model, processorName, stageName);
		String viewProcessorBean = processorStage.getViewProcessorBean();
		if (StringUtil.isBlank(viewProcessorBean)) {
			return;
		}
		ModelViewProcessor viewProcessor = SpringUtil
				.getBean(viewProcessorBean);
		viewProcessor.process(model, context, writer);
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	public void processParameter(String modelId, String processorName,
			String stageName, Context context) {
		Object model = getModel(modelId);
		ModelProcessorStage processorStage = container.getModelProcessorStage(
				model, processorName, stageName);
		String builderBean = processorStage.getParameterBuilderBean();
		if (StringUtil.isBlank(builderBean)) {
			return;
		}
		ParameterBuilder builder = SpringUtil.getBean(builderBean);
		builder.buildParameter(model, context);
	}

	public List<ModelLoader> getModelLoaders() {
		return container.getModelLoaders();
	}

}
